package com.baidu.oauth.demo;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Build;
import android.support.annotation.RequiresApi;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.view.View;

import com.baidu.oauth.demo.utils.Utility;
import com.baidu.oauth.demo.utils.ViewUtility;
import com.baidu.oauth.demo.view.CustomAlertDialog;
import com.baidu.oauth.demo.zxing.activity.CaptureActivity;
import com.baidu.oauth.sdk.auth.AuthInfo;
import com.baidu.oauth.sdk.auth.BdOauthSdk;
import com.baidu.oauth.sdk.auth.BdSsoHandler;
import com.baidu.oauth.sdk.callback.BdOauthCallback;
import com.baidu.oauth.sdk.callback.QrCodeCallback;
import com.baidu.oauth.sdk.callback.QrLoginStatusCheckCallback;
import com.baidu.oauth.sdk.dto.BdOauthDTO;
import com.baidu.oauth.sdk.result.BdOauthResult;
import com.baidu.oauth.sdk.result.QrCodeResult;
import com.baidu.oauth.sdk.result.QrLoginStatusCheckResult;
import com.baidu.oauth.sdk.utils.Log;
import com.baidu.oauth.sdk.view.ListDialog;

import java.util.UUID;

public class MainActivity extends AppCompatActivity implements View.OnClickListener {

    private BdSsoHandler bdSsoHandler;
    private static final String TAG = "oauth_demo";
    private static final int REQUEST_QR_LOGIN = 2001;
    private static final int PERMISSIONS_REQUEST_CAMERA = 0;
    private static int scanQrCodeType;



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        findViewById(R.id.oauth_login).setOnClickListener(this);
        findViewById(R.id.oauth_qr_login_show).setOnClickListener(this);
        findViewById(R.id.oauth_qr_login_scan).setOnClickListener(this);


        initSdk();

    }

    @Override
    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.oauth_login:
                oauthLogin();
                break;
            case R.id.oauth_qr_login_show:
                oauthQrLoginShow();
                break;
            case R.id.oauth_qr_login_scan:
                oauthQrLoginScan();
                break;

            default:
                break;

        }

    }



    private void oauthQrLoginScan() {
        final ListDialog listDialog = new ListDialog(this);
        ListDialog.ItemClickListener itemClickListener = new ListDialog.ItemClickListener() {
            @Override
            public void onClick(ListDialog.Item item) {
                listDialog.dismiss();
                scanQrCodeType = (int) item.getExtra();
                // 二维码识别过程，这部分产品线可以自己实现
                if (Build.VERSION.SDK_INT >= 23 && checkSelfPermission(Manifest.permission.CAMERA)
                        != PackageManager.PERMISSION_GRANTED) {
                    getCameraPermission(MainActivity.this);
                } else {
                    Intent intent = new Intent(MainActivity.this, CaptureActivity.class);
                    startActivityForResult(intent, REQUEST_QR_LOGIN);
                }
            }
        };

        listDialog.addItem(new ListDialog.Item("调起手百授权或者引导下载手百", itemClickListener, 0));
        listDialog.addItem(new ListDialog.Item("调起手百或者Web登录授权", itemClickListener, 1));
        listDialog.addItem(new ListDialog.Item("直接调起Web登录授权", itemClickListener, 2));
        listDialog.show();

    }


    private void oauthLogin() {
        bdSsoHandler = new BdSsoHandler(MainActivity.this);
        BdOauthDTO bdOauthDTO = new BdOauthDTO();
        bdOauthDTO.oauthType = BdOauthDTO.OAUTH_TYPE_BOTH;
        // 重定向后会携带state参数，建议开发者利用state参数来防止CSRF攻击
        bdOauthDTO.state = UUID.randomUUID().toString();
        bdSsoHandler.authorize(bdOauthDTO, new BdOauthCallback() {
            @Override
            public void onSuccess(BdOauthResult result) {
                ViewUtility.showToast("code = " + result.getCode() + " state = " + result.getState());

            }

            @Override
            public void onFailure(BdOauthResult result) {
                ViewUtility.showToast("result code = " + result.getResultCode() + " msg = " + result.getResultMsg());

            }
        });

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        String result = data == null ? "" : data.getStringExtra("extra_oauth_result_json");
        Log.d(TAG, "on Activity Result data is " + result);
        // 处理授权回调
        if (bdSsoHandler != null) {
            bdSsoHandler.authorizeCallBack(requestCode, resultCode, data);
        }

        if (data != null) {
            // 处理扫码pass sdk二维码通过手百授权登录设备，同时返回调用方accessToken信息
            handlerQrLogin(requestCode, resultCode, data);
        }
    }


    // 二维码扫码回调 产品线调用时通过自身APP获取到二维码内容
    private void handlerQrLogin(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_QR_LOGIN) {
            if (resultCode == Activity.RESULT_OK) {
                String qrCode = data.getStringExtra(CaptureActivity.BUNDLE_SCAN_RESULT_TEXT);
                Log.d(TAG, "qrCode is " + qrCode);
                if (BdOauthSdk.getInstance().isQrLoginSchema(qrCode)) {
                    bdSsoHandler = new BdSsoHandler(MainActivity.this);

                    if (scanQrCodeType == 0) {
                        bdSsoHandler.authorize(qrCode, new BdOauthCallback() {
                            @Override
                            public void onSuccess(BdOauthResult result) {
                                ViewUtility.showToast("code = " + result.getCode());
                            }

                            @Override
                            public void onFailure(BdOauthResult result) {
                                ViewUtility.showToast("code = " + result.getResultCode()
                                        + " msg = " + result.getResultMsg());
                            }
                        });
                    } else if (scanQrCodeType == 1){
                        bdSsoHandler.authorize(qrCode, new BdOauthCallback() {
                            @Override
                            public void onSuccess(BdOauthResult result) {
                                ViewUtility.showToast("code = " + result.getCode());
                            }

                            @Override
                            public void onFailure(BdOauthResult result) {
                                ViewUtility.showToast("code = " + result.getResultCode()
                                        + " msg = " + result.getResultMsg());
                            }
                        }, BdSsoHandler.GUID_TYPE_BAIDUAPP_AND_WEB);
                    } else if (scanQrCodeType == 2){
                        bdSsoHandler.authorize(qrCode, new BdOauthCallback() {
                            @Override
                            public void onSuccess(BdOauthResult result) {
                                ViewUtility.showToast("code = " + result.getCode());
                            }

                            @Override
                            public void onFailure(BdOauthResult result) {
                                ViewUtility.showToast("code = " + result.getResultCode()
                                        + " msg = " + result.getResultMsg());
                            }
                        }, BdSsoHandler.GUID_TYPE_ONLY_WEB);
                    } else {
                        Log.i("unsupported login guide type");
                    }
                } else {
                    Log.i("other qr code");
                }
            }
        }
    }


    private void oauthQrLoginShow() {

        String state = UUID.randomUUID().toString();
        BdOauthSdk.getInstance().getQrCode(state, new QrCodeCallback() {
            @Override
            public void onSuccess(QrCodeResult result) {

                Bitmap bitmap = Utility.createBitmap(result.url);
                final QrCodeImageDialog dialog = new QrCodeImageDialog(MainActivity.this);
                dialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
                    @Override
                    public void onCancel(DialogInterface dialogInterface) {
                        BdOauthSdk.getInstance().stopLoginStatusCheck();
                    }
                });

                dialog.setQrCodeBitmap(bitmap);
                dialog.show();
                BdOauthSdk.getInstance().qrLoginStatusCheck(result.channelId, new QrLoginStatusCheckCallback() {

                    @Override
                    public void onScanQrCodeDone(QrLoginStatusCheckResult result) {
                        ViewUtility.showToast("qr oauth login scan qr code done");
                    }

                    @Override
                    public void onSuccess(QrLoginStatusCheckResult result) {

                        ViewUtility.showToast("code = " + result.getCode() + " state = " + result.getState());
                        dialog.dismiss();
                        BdOauthSdk.getInstance().stopLoginStatusCheck();


                    }

                    @Override
                    public void onFailure(QrLoginStatusCheckResult result) {
                        ViewUtility.showToast("code = " + result.getResultCode() + " msg = " + result.getResultMsg());
                        BdOauthSdk.getInstance().stopLoginStatusCheck();

                    }

                    @Override
                    public void onStart() {
                        Log.i(TAG, "qr oauth login start");

                    }

                    @Override
                    public void onFinish() {
                        Log.i(TAG, "qr oauth login finish");
                    }
                });


            }

            @Override
            public void onFailure(QrCodeResult result) {

                ViewUtility.showToast(String.format("%s(%d)", result.getResultMsg(), result.getResultCode()));

            }

            @Override
            public void onStart() {

            }

            @Override
            public void onFinish() {

            }
        });


    }


    /**
     * 初始化SDK
     */
    private void initSdk() {
        String redirectUrl = "https://passport.baidu.com";
        String scope = "basic";
        String appKey = "XXIlWUS6GYbkDIG0OAkGzhEt";
        AuthInfo authInfo = new AuthInfo(MainActivity.this, appKey, redirectUrl, scope);
        BdOauthSdk.init(authInfo);
        authInfo.isDebug(true);
    }


    // 获取相机权限，扫码功能，必要
    @RequiresApi(api = Build.VERSION_CODES.M)
    private void getCameraPermission(Context context) {
        if (shouldShowRequestPermissionRationale(Manifest.permission.CAMERA)) {

            final CustomAlertDialog requestCameraPermission = new CustomAlertDialog(context);
            requestCameraPermission.setMessageText(
                    getString(R.string.permission_camera_get_camera_permission_msg));
            requestCameraPermission.setPositiveBtn(getString(R.string.permission_ok),
                    new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            ViewUtility.dismissDialog(MainActivity.this, requestCameraPermission);
                            MainActivity.this.requestPermissions(new String[]{Manifest.permission.CAMERA},
                                    PERMISSIONS_REQUEST_CAMERA);
                        }
                    });
            requestCameraPermission.setNegativeBtn(getString(R.string.permission_cancel),
                    new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            ViewUtility.dismissDialog(MainActivity.this, requestCameraPermission);
                        }
                    });
            requestCameraPermission.show();

        } else {
            requestPermissions(new String[]{Manifest.permission.CAMERA}, PERMISSIONS_REQUEST_CAMERA);
        }
    }


}
